
/* 
				ARC - Automated Resource Classifier
This is a C program to analyze FASTA format protein (with or without sequence) files and to functionally classify proteins on the basis of keywords present in FASTA header.

It takes FASTA protein sequence file as input.

It requires the name of the output file from the user.

It needs a code from the user for gene symbol search.

It calculates the frequency of 20 standard aminoacids in the input file.

It classifies each protein into one of the eight basic functional classes. For that, it starts searching the FASTA header information (annotation text) for the presence of keywords of its library.

-----------------------------------------------------------|
Class Symbol		-	Class			   |	
-----------------------------------------------------------|
C 				Cell Wall and Cell Membrane|
				and Transport		   |
							   |
D				Cell Division		   |
							   |
I				Information (Replication   |	
				Transcription, Translation)|
							   |
L				Translocation and Secretion|
							   |
R				Stress			   |
							   |
S				Signal and Communication   |
							   |
M				Metabolism		   |
-----------------------------------------------------------|

It needs the keyword library (26 ascii files having keyword entries) in a subfolder named 'library' (case should match, for ex, 'Library' will not work. So please use the default name 'library') in the same folder where the executable version of this program exists.

The keyword library has 400 keyword entries.

The user can browse keyword files and include or delete any keyword with the associated class symbol (except 'H' )separated by an '_' symbol. For example, the entry 'RNA_I' where RNA is the keyword and I is for the information class, can be included into the library in the rkey.txt file.

The class symbol "H" is reserved for hypothetical proteins by the program.
To make virus and spore related proteins explicitly as Unclassified, the keyword library has reserved the symbol 'U' also. So, the class symbol "U" also, like "H", is NOT available to the user.

The character length of the class symbol must be lesser than 10 ie., the user can use a multicharacter class for ex, "Cell" or "CELL" or "cELl" (here the character lenght is 4), instead of a single character 'C'.

While including new keywords into the library, if the user feels that ARC should find if and only if, it is a separate word in a FASTA header (ie. the keyword is not a substring of any word in FASTA header), that keyword should be included with a prefix '$' and a suffix ' ' (white space).

For example, 'RNA' is a keyword user wants to enter but the word in FASTA header 'internal' has the substring rna and will get surprised by the classification internal membrane as Information class. To avoid this, the user can enter the keyword as '$RNA _I' (note the white space between RNA and underscore symbol. The keyword entry has been given within single quotes, so the user does and should not give them!).

In the output file, 'H' symbol is to denote that the protein is a hypothetical one and the 'U' symbol denotes that AuRA is unable to classify it.

The output file has tab delimited values for the user to conveniently open in Excel, Openoffice.org, etc., So it is better to give the output file name with appropriate extension (for ex. .csv for Openoffice.org).
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>

FILE *f_faafile;
FILE *f_outputfile;

char * GetGeneline(char *geneline, int *len)
{
int l = 0;
char c;
	while(((c = getc(f_faafile)) != EOF) && (c != '\n') && (c != '\r') && (c != '\f'))
	{
		geneline = realloc(geneline, (++l) * sizeof(char));
		geneline[l-1] = c;
	}
	geneline = realloc(geneline, (++l) * sizeof(char));
	geneline[l - 1] = '\0';
	*len = l;
	return(geneline);
}

void ClassifyGeneline(char *genelinedum, char *class, char *keywordfile,int Mkey)
{
int i,j,k;
int boolean;
char c;
FILE *f_keyfile;
boolean = 1;
for(i = 0; ((i < strlen(genelinedum)) && (boolean)); ++i)
{
	if(!isalpha(genelinedum[i])) continue;
	keywordfile[strlen(keywordfile) - 8] = tolower(genelinedum[i]);
	f_keyfile = fopen(keywordfile,"r");
	if(f_keyfile == NULL)
	{
		printf("\nFile Open Error\nUnable To Open %s\n",keywordfile);
		exit(0);
	}
	while(((c = getc(f_keyfile)) != EOF) && (boolean))
	{
		if(c != '\n') continue;
		j = i;
		while(((c = tolower(getc(f_keyfile))) != EOF) && (boolean) && (genelinedum[j] != '\0'))
		{
			if((c == '$') && (isalpha(genelinedum[j-1]))) break;
			if(c == '$') continue;
			if(c == tolower(genelinedum[j++])) continue;
			if(c == '_')	 
			{
				fscanf(f_keyfile,"%s",class);
				if(((class[0] != 'M') && (Mkey)) || ((!Mkey) && (class[0] != 'U')))
					boolean = 0;
			}
			else break;
		}
	}
	fclose(f_keyfile);			
}
	if(((strstr(genelinedum,"hypothetic")) || (strstr(genelinedum,"unknown cds")) || (strstr(genelinedum,"conserved archael protein")) || (strstr(genelinedum,"conserved protein")) || (strstr(genelinedum,"conserved crenarchael protein")) || (strstr(genelinedum,"putative enzyme")) || (strstr(genelinedum,"uncharacterized protein")) || (strstr(genelinedum,"uncharacterized conserved protein")) || (strstr(genelinedum,"putative conserved protein")) || (strstr(genelinedum,"unknown protein")) || (strstr(genelinedum,"function unknown")) || (strstr(genelinedum,"unknown function"))) && (class[0] == 'U'))

	class[0] = 'H';

}

void getNotkeySynthkeyRibokey(char *genelinedum, char *class, int *Notkey, int *Synthkey, int *NonRibokey, int *Ribokey)
{
	if((strstr(genelinedum," not")) || (strstr(genelinedum," non")))
		*Notkey = 1;
	if((strstr(genelinedum,"synthesis")) || (strstr(genelinedum,"synthetic")))
		*Synthkey = 1;
	if((strstr(genelinedum,"nonribosom")) || (strstr(genelinedum,"non ribosom")))
		*NonRibokey = 1;
	if(strstr(genelinedum,"ribosom"))
		*Ribokey = 1;

}


char * GetGeneSymbolSynonym(char *genelinedum, char *symbolfilename, int *symtag)
{
	FILE *f_symboldict;
	char c,genesymbol[25],genesymboldum[25], *pchar;
	int i,len, k;
	for(i = 1; ((i < strlen(genelinedum)) && (*symtag == 0)); ++i)
	{
		if((!isalpha(genelinedum[i])) || (isalpha(genelinedum[i-1])))
			continue;
		symbolfilename[strlen(symbolfilename)-11] = genelinedum[i];
		f_symboldict = fopen(symbolfilename,"r");
		if(f_symboldict == NULL)
		{
			printf("File Open Error\nThe File %s Is Not Found\n",symbolfilename);
			exit(0);
		}
		while(((c = getc(f_symboldict)) != EOF) && (!(*symtag)))
		{
			if(c != '\n') continue;
			fscanf(f_symboldict,"%s",genesymbol);
			for(k = 0; k < strlen(genesymbol); ++k)
				if(genesymbol[k] == '-') genesymbol[k] = ' ';
				else genesymbol[k] = tolower(genesymbol[k]);
			strcat(strcat(strcpy(genesymboldum," "),genesymbol)," ");
			if(strstr(genelinedum,genesymboldum) )
				*symtag = 1;

		}

		if((*symtag) == 0)
		{
			fclose(f_symboldict);
			continue;
		}
		len = 0;
		genelinedum = malloc(0);
		while(((c = getc(f_symboldict)) != EOF) && (c != '\n') && (c != '\r'))
		{
			genelinedum = realloc(genelinedum,++len);
			genelinedum[len - 1] = (((c == '-') || (c == '/') || (c == '_'))?' ':tolower(c));		
		}
		genelinedum = realloc(genelinedum,len+2);
		genelinedum[len] = ' ';
		genelinedum[len + 1] = '\0';
		fclose(f_symboldict);
		return genelinedum;
	}
if(!(*symtag))
	return genelinedum;
}

main()
{
FILE *f_symbolfile;
DIR *libdir, *symdir;
char faafile[250];
char c, *pchar;
char *geneline;
char *genelinedum;
char outputfile[250];
char symboldirname[300];
char libdirname[300];
char class[11];
char yes;
int boolean;
int i,j,n,len;
int Synthkey, Notkey, NonRibokey, Ribokey;
int sqbracket;
int filebool = 1;
int total;
int proteincount;
int symtag;
char keyfile[350];

printf("\n***************************************************\n");
printf("\nPlease Note That All The File Names And Directory Names Should Be Less Than 300 Characters Length\n");
printf("\n***************************************************\n");

filebool = 1;
while(filebool)
{
	printf("\nEnter The Name (Full Path) of Directory That Has All Keyword Files\n\t");
	scanf("%s",libdirname);		
	libdir = opendir(libdirname);
	if(libdir == NULL)
		printf("\nThe Directory %s Is NOT Opened\nCheck The Name And Path And Please Try Again\n\t");
	else
	{
		closedir(libdir);
		if(libdirname[strlen(libdirname) - 1] != '/')
			strcat(libdirname,"/");
		strcat(strcpy(keyfile,libdirname)," key.txt");
		filebool = 0;
	}
}
while(!filebool)
{
	printf("\nEnter The Name (Full Path) of The Directory Where Gene Symbol Synonym Files Are Kept\n\t");
	scanf("%s",symboldirname);
	symdir = opendir(symboldirname);
	if(symdir == NULL)
		printf("\n%s NOT Opened\nPlease Try Again\n",symboldirname);
	else
		filebool = 1;
}
if(symboldirname[strlen(symboldirname) - 1] != '/')
	strcat(symboldirname,"/");
strcat(symboldirname," symbol.txt");
do
{

	filebool = 1;
	while(filebool)
	{
		printf("\nEnter The Input File Name Which Has The Fasta Format Protein Sequence\n");
		scanf("%s",faafile);
		f_faafile = fopen(faafile,"r");
		if(f_faafile == NULL)
			printf("\nError!!! Input File NOT Opened\nTry Again\n\t");
		else
			filebool = 0;
	}
	filebool = 1;
	while(filebool)
	{
		printf("\nEnter The Output File Name To Store The Classification Result\n\t");
		scanf("%s",outputfile);
		f_outputfile = fopen(outputfile,"w");
		if(f_outputfile == NULL)
		{
			printf("\nOutput File Is NOT Opened To Write\nCheck Whether You Have Enough Memory Space or Write Permission\nProgram Terminated\n");
			exit(0);			
		}
		else
			filebool = 0;
	}
	
	printf("%s is Being Processed into %s............\n\n\n\n",faafile,outputfile);
	fprintf(f_outputfile,"Protein\tFunctional Class");
	proteincount = 0;
	while((c = getc(f_faafile)) != EOF)
	{
		if(c != '>') continue;
		proteincount++;
		geneline = malloc(0);
		genelinedum = malloc(0);
		geneline = GetGeneline(geneline, &len);
		genelinedum = malloc(len + 3);		
		strcat(strcat(strcpy(genelinedum," "),geneline)," ");
		for(i = 0; i < strlen(genelinedum) - 1; ++i)
		{
			((genelinedum[i] == '/') || (genelinedum[i] == '-') || (genelinedum[i] == '_'))? genelinedum[i] = ' ': 0;
			genelinedum[i] = tolower(genelinedum[i]);
		}
		total = 0;
		while(((c = tolower(getc(f_faafile))) != EOF) && (c != '>'));
		fseek(f_faafile,-2L,1);
		symtag = Synthkey = Notkey = NonRibokey = Ribokey = 0;
		strcpy(class,"U");

		getNotkeySynthkeyRibokey(genelinedum, class, &Notkey, &Synthkey, &NonRibokey, &Ribokey);
		if(NonRibokey) 
			class[0] = 'M';
		else if((Ribokey) && (!Notkey))
			class[0] = 'I';
		else if(Notkey)
			class[0] = 'U';
		else
		{
			ClassifyGeneline(genelinedum,class,keyfile,Synthkey);
			if(class[0] == 'U')
			{
				if(Synthkey)
					class[0] = 'M';
				else
				{
					genelinedum = GetGeneSymbolSynonym(genelinedum,symboldirname,&symtag);
					if(symtag)
					{
						getNotkeySynthkeyRibokey(genelinedum,class,&Notkey,&Synthkey,&NonRibokey, &Ribokey);
						if(NonRibokey) 
							class[0] = 'M';
						else if((Ribokey) && (!Notkey))
							class[0] = 'I';
						else if(Notkey)
							class[0] = 'U';
						else
						{
							ClassifyGeneline(genelinedum,class,keyfile,Synthkey);
							if((class[0] == 'U') && (Synthkey))
								class[0] = 'M';
						}
					}
				}
			}
		}
		fprintf(f_outputfile,"\n>%s\t%s",geneline,class);
	}
	fclose(f_faafile);
	fclose(f_outputfile);
	printf("\nTotal Number of Proteins Processed : %d",proteincount);
	printf("\n%s Has Been Successfully Classified into %s\n",faafile,outputfile);		
	printf("\nDo You Wish To Continue ( Y/N )? : ");
	yes = tolower(getchar());
	yes = tolower(getchar());

}while(yes == 'y');
}
